#include "Input.h"

namespace mdd{
    int Input::addConnection(std::shared_ptr<IOutput> output)
    {
        _output = output;
        return 1;
    }
    int Input::removeConnection(std::shared_ptr<IOutput> output)
    {
        _output = nullptr;
        return 0;
    }
    Input::Input(const std::string& type, int appendix, const json& default_value,
                 const std::function<bool(const json&)>& verification) {
        _type = type;
        _value = default_value;
        _verification = std::move(verification);
        _prefix = "";
        _appendix = appendix;
    }

    std::string Input::setType(std::string type){
        _type = type;
        return getID();
    }
    std::string Input::getType(){
        return  _type;
    }
    std::string Input::getID(){
        return _prefix + "/" + _type + std::to_string(_appendix);
    }

    std::string Input::setPrefix(std::string prefix){
        _prefix = prefix;
        return getID();
    }

    std::string Input::setAppendix(int appendix){
        _appendix = appendix;
        return  getID();
    }

    state Input::getState() {
        if (_output == nullptr) {
            return state::UNCHANGED;
        }
        else {
            return _output->getState();
        }
    }

    void Input::resetState() {

    }

    const json& Input::getValue() {
        if(_output == nullptr){
            return _value;
        } else{
            return _output->getValue();
        }
    }

    std::shared_ptr<IOutput> Input::getConnection() { return _output; }

    json& Input::setDefaultValue(){
        return _value;
    }

    bool Input::verify(const json & data){
        return _verification(data);
    }

    bool Input::connect(std::shared_ptr<IOutput> output){
        if (verify(output->getValue())) {
            _output = output;
            _output->addConnection(std::make_shared<Input>((*this)));
            return true;
        }
        return false;
    }
}